@@ -129,8 +129,12 @@ CORBA_Object_release_cb (ORBit_RootObject robj)
 		ORBit_RootObject_release_T (obj->adaptor_obj);
 	}
 
-	if (obj->connection)
+	
+	if (obj->connection) {
+		LINK_MUTEX_UNLOCK (ORBit_RootObject_lifecycle_lock);
 		giop_connection_unref (obj->connection);
+		LINK_MUTEX_LOCK   (ORBit_RootObject_lifecycle_lock);
+	}
 
 	p_free (obj, struct CORBA_Object_type);
 }
@@ -164,7 +168,7 @@ ORBit_objref_find (CORBA_ORB   orb,
 		   GSList     *profiles)
 {
 	CORBA_Object retval = CORBA_OBJECT_NIL;
-	struct CORBA_Object_type fakeme = {{0}};
+	struct CORBA_Object_type fakeme = {{NULL}};
 
 	fakeme.orb = orb;
 	fakeme.type_qid = g_quark_from_string (type_id);
@@ -207,6 +211,12 @@ ORBit_objref_find (CORBA_ORB   orb,
 	return retval;
 }
 
+static gboolean
+is_localhost (const char *host)
+{
+	return (host && !strcmp (link_get_local_hostname (), host)) ? TRUE : FALSE;
+}
+
 /**
  * ORBit_objref_get_proxy:
  * @obj: the local object
@@ -263,9 +273,10 @@ ORBit_try_connection_T (CORBA_Object obj)
 		break;
 	case LINK_DISCONNECTED:
 		/* Have a go at reviving it */
-		dprintf (MESSAGES, "re-connecting dropped cnx %p", cnx);
+		dprintf (MESSAGES, "re-connecting dropped cnx %p: ", cnx);
 		if (giop_connection_try_reconnect (GIOP_CONNECTION (cnx)) == LINK_CONNECTED)
 			retval = TRUE;
+		dprintf (MESSAGES, retval ? "connected\n" : "not connected\n" );
 		break;
 	}
 
@@ -276,6 +287,21 @@ ORBit_try_connection_T (CORBA_Object obj)
 	return retval;
 }
 
+GIOPConnection *
+ORBit_object_peek_connection (CORBA_Object obj)
+{
+	GIOPConnection *cnx;
+
+	OBJECT_LOCK (obj);
+	
+	if ((cnx = obj->connection))
+		giop_connection_ref (cnx);
+
+	OBJECT_UNLOCK (obj);
+
+	return cnx;
+}
+
 GIOPConnection *
 ORBit_object_get_connection (CORBA_Object obj)
 {
@@ -287,9 +313,15 @@ ORBit_object_get_connection (CORBA_Object obj)
 	gboolean is_ssl = FALSE;
 	GIOPVersion iiop_version = GIOP_1_2;
 	GIOPConnection *cnx = NULL;
+	gboolean unix_socket_enabled = FALSE;
+	gboolean ipv4_ipv6_enabled = FALSE;
+	gboolean unix_socket_failed = FALSE;
 
 	OBJECT_LOCK (obj);
 
+	unix_socket_enabled = ORBit_proto_use ("UNIX");
+	ipv4_ipv6_enabled = (ORBit_proto_use ("IPv4") || ORBit_proto_use ("IPv6"));
+
 	if (obj->connection) {
 		if (ORBit_try_connection_T (obj)) {
 			cnx = obj->connection;
@@ -318,10 +350,18 @@ ORBit_object_get_connection (CORBA_Object obj)
 		if (IOP_profile_get_info (obj, pinfo, &iiop_version, &proto,
 					  &host, &service, &is_ssl, tbuf)) {
 
+			if (unix_socket_failed && ipv4_ipv6_enabled && is_localhost (host))
+				continue;
+
 			obj->connection = giop_connection_initiate (
 				obj->orb, proto, host, service,
 				is_ssl ? LINK_CONNECTION_SSL : 0, iiop_version);
 
+			if (!obj->connection && unix_socket_enabled && ipv4_ipv6_enabled) {
+				if (!strcmp (proto,"UNIX")) 
+					unix_socket_failed = TRUE;
+			}
+	
 			if (obj->connection && ORBit_try_connection_T (obj)) {
 				obj->object_key = objkey;
 				obj->connection->orb_data = obj->orb;
@@ -346,6 +386,7 @@ ORBit_handle_location_forward (GIOPRecvBuffer *buf,
 {
 	GIOPConnection *retval = NULL;
 	GSList         *profiles = NULL;
+	GIOPConnection *old_connection;
 
 	if (ORBit_demarshal_IOR (obj->orb, buf, NULL, &profiles))
 		goto out;
@@ -353,8 +394,15 @@ ORBit_handle_location_forward (GIOPRecvBuffer *buf,
 	OBJECT_LOCK (obj);
 	IOP_delete_profiles (obj->orb, &obj->forward_locations);
 	obj->forward_locations = profiles;
+
+	/* We need to clear the connection because 
+	   forwarding results in a different connection */
+	old_connection = obj->connection;
+	obj->connection = NULL;
 	OBJECT_UNLOCK (obj);
 
+	giop_connection_unref (old_connection);
+
 	retval = ORBit_object_get_connection (obj);
 
  out:
